@@ -45,7 +45,7 @@ and in_value =
   | Fun     of (string * string * value option) list *
                env * env * Lang_values.term
   | FFI     of (string * string * value option) list *
-               env * env * (env -> value)
+               env * env * (env -> kind -> value)
 
 (** Get a string representation of a value. *)
 val print_value : value -> string
@@ -69,15 +69,22 @@ type doc_flag =
   | Deprecated (** The plugin should not be used. *)
   | Experimental (** The plugin should not considered as stable. *)
 
-(** Add an builtin to the language. *)
+(** Add an builtin to the language, high-level version for functions. *)
 val add_builtin :
   category:string ->
   descr:string ->
   ?flags:doc_flag list ->
   string ->
-  proto -> kind -> (env -> value) ->
+  proto -> kind -> (env -> kind -> value) ->
   unit
 
+(** Add an builtin to the language, more rudimentary version. *)
+val add_builtin_base :
+  category:string ->
+  descr:string ->
+  ?flags:doc_flag list ->
+  string -> in_value -> kind -> unit
+
 (** Category of an operator. *)
 type category =
   | Input (** Input. *)
@@ -85,7 +92,9 @@ type category =
   | TrackProcessing (** Operations on tracks (e.g. mixing, etc.). *)
   | SoundProcessing (** Operations on sound (e.g. compression, etc.). *)
   | VideoProcessing (** Operations on video. *)
+  | MIDIProcessing (** Operations on MIDI. *)
   | Visualization (** Visializations of the sound. *)
+  | SoundSynthesis (** Synthesis. *)
 
 (** Get a string representation of a [doc_flag]. *)
 val string_of_flag : doc_flag -> string
@@ -96,7 +105,7 @@ val add_operator :
   descr:string ->
   ?flags:doc_flag list ->
   string ->
-  proto -> (env -> Source.source) ->
+  proto -> (env -> kind -> Source.source) ->
   unit
 
 (** {2 Manipulation of values} *)
@@ -161,7 +170,8 @@ val list : value list -> value
 val source : Source.source -> value
 val request : Request.raw Request.t option -> value
 val product : value -> value -> value
-val val_fun : (string * string * value option) list -> (env -> value) -> value
+val val_fun :
+      (string * string * value option) list -> (env -> kind -> value) -> value
 
 (** Specialized builder for constant functions.
   * It is slightly less opaque and allows the printing of the closure
