@@ -36,7 +36,26 @@ let string_of_category = function
 
 let add_builtin ~cat ~descr ?flags name proto ret_t f =
   Lang.add_builtin ~category:(string_of_category cat)
-    ~descr ?flags name proto ret_t f
+    ~descr ?flags name proto ret_t (fun p _ -> f p)
+
+let () =
+  Lang.add_builtin_base
+    ~category:(string_of_category Liq)
+    ~descr:"Liquidsoap version string."
+    "liquidsoap.version"
+    (Lang.String Configure.version)
+    Lang.string_t ;
+  List.iter
+    (fun (name,kind,str) ->
+       Lang.add_builtin_base
+         ~category:(string_of_category Liq)
+         ~descr:(Printf.sprintf "Liquidsoap's %s directory." kind)
+         ("configure."^name)
+         (Lang.String str)
+         Lang.string_t)
+    [ ("libdir", "library", Configure.libs_dir) ;
+      ("rundir", "PID file", Configure.rundir) ;
+      ("logdir", "logging", Configure.logdir) ]
 
 (** Liquidsoap stuff *)
 
@@ -385,11 +404,11 @@ let () =
        let i = ref (Lang.to_string (List.assoc "" p)) in
        let get  =
          Lang.val_fun []
-         (fun p -> Lang.string !i)
+         (fun p _ -> Lang.string !i)
        in
        let set =
          Lang.val_fun ["","",None]
-         (fun p ->
+         (fun p _ ->
              let v = List.assoc "" p in
              i := Lang.to_string v ;
              Lang.unit)
@@ -1344,7 +1363,7 @@ let () =
                         with _ ->
                           raise (Var.Invalid_value
                                    (s ^ " is not a string")));
-         Lang.val_fun [] (fun p -> Lang.string !v))
+         Lang.val_fun [] (fun p _ -> Lang.string !v))
 
 let () =
   add_builtin "interactive.float" ~cat:Interaction
@@ -1366,7 +1385,7 @@ let () =
                         with _ ->
                           raise (Var.Invalid_value
                                    (s ^ " is not a float")));
-         Lang.val_fun [] (fun p -> Lang.float !v))
+         Lang.val_fun [] (fun p _ -> Lang.float !v))
 
 let () = (* TODO do not print when daemonized *)
   add_builtin "print" ~cat:Interaction ~descr:"Print on standard output."
