@@ -34,22 +34,19 @@ object (self)
   method abort_track = source#abort_track
 
   val mutable last = 0.
-  (* failed means that we're not currently streaming *)
-  val mutable failed = true
+  val mutable in_track = false
 
-  method delay_ok = (Unix.time ()) -. last >= delay
-  method is_ready = (not failed) || (self#delay_ok && source#is_ready)
+  method private delay_ok = (Unix.time ()) -. last >= delay
+  method is_ready = in_track || (self#delay_ok && source#is_ready)
 
-  method get_frame buf =
-    match failed,self#delay_ok with
-      | false, _ ->
-          source#get buf ;
-          if Frame.is_partial buf then
-            ( failed <- true ; last <- Unix.time () )
-      | true, true ->
-          failed <- false ;
-          source#get buf
-      | true, false -> ()
+  method private get_frame buf =
+    source#get buf ;
+    in_track <- true ;
+    (* The current track ends. *)
+    if Frame.is_partial buf then begin
+      in_track <- false ;
+      last <- Unix.time ()
+    end
 
 end
 
@@ -63,8 +60,8 @@ let () =
     ~category:Lang.TrackProcessing
     ~descr:("Prevents the child from being ready again too fast after "^
             "a end of track")
-    (fun p ->
+    (fun p _ ->
        let f n = Lang.assoc "" n p in
        let d = Lang.to_float (f 1) in
        let s = Lang.to_source (f 2) in
-         ((new delay s d):>source))
+         new delay s d)
