@@ -101,7 +101,7 @@ object (self)
 
   val mutable activation = []
 
-  method wake_up activator =
+  method private wake_up activator =
     activation <- (self:>source)::activator ;
     List.iter
       (fun { transition = transition ; source = s } ->
@@ -111,7 +111,7 @@ object (self)
            transition)
       cases
 
-  method sleep =
+  method private sleep =
     List.iter
       (fun { transition = transition ; source = s } ->
          s#leave ~dynamic:true (self:>source) ;
@@ -124,7 +124,7 @@ object (self)
 
   method is_ready = need_eot || selected <> None || self#cached_select <> None
 
-  method get_frame ab =
+  method private get_frame ab =
     (* Choose the next child to be played.
      * [forget] tells that the current child has finished its track,
      * in which case a transition does not make sense and would actually start
@@ -263,7 +263,7 @@ let common = [
 ]
 
 let default_transition =
-  Lang.val_fun [ "","x",None ; "","y",None ] (fun e -> List.assoc "y" e)
+  Lang.val_fun [ "","x",None ; "","y",None ] (fun e _ -> List.assoc "y" e)
 
 let extract_common p l =
   let ts =
@@ -346,7 +346,7 @@ let () =
       ~descr:("At the beginning of a track, select the first source "^
               "whose predicate is true.")
       (common@proto)
-      (fun p ->
+      (fun p _ ->
          let children =
            List.map
              (fun p ->
@@ -377,7 +377,7 @@ let () =
                           (List.assoc "single" p,
                            "there should be exactly one flag per children"))
          in
-           ((new lang_switch ~replay_meta ts children):>source))
+           new lang_switch ~replay_meta ts children)
 
 (** Fallback selector: switch to the first ready source. *)
 class fallback ?replay_meta mode children =
@@ -407,7 +407,7 @@ let () =
     Lang.add_operator "fallback" ~category:Lang.TrackProcessing
       ~descr:"At the beginning of each track, select the first ready child."
       (common@proto)
-      (fun p ->
+      (fun p _ ->
          let children = Lang.to_source_list (List.assoc "" p) in
          let replay_meta,ts,tr = extract_common p (List.length children) in
          let children =
@@ -415,7 +415,7 @@ let () =
              (fun t s -> { transition = t ; cur_meta = None ; source = s })
              tr children
          in
-           ((new fallback ~replay_meta ts children):>source))
+           new fallback ~replay_meta ts children)
 
 (** Random switch *)
 exception Found of child
@@ -454,15 +454,13 @@ object
 end
 
 let () =
-    Lang.add_operator "random" ~category:Lang.TrackProcessing
+  let add name strict descr weight_descr =
+    Lang.add_operator name ~descr ~category:Lang.TrackProcessing
       (common @
        [ "weights", Lang.list_t Lang.int_t, Some (Lang.list []),
-         Some "Weights of the children in the choice." ;
-         "strict", Lang.bool_t, Some (Lang.bool false),
-         Some "Do not use random but cycle over the uniform distribution." ;
+         Some weight_descr ;
          "", Lang.list_t Lang.source_t, None, None ])
-      ~descr:"At the beginning of every track, select a random ready child."
-      (fun p ->
+      (fun p _ ->
          let children = Lang.to_source_list (List.assoc "" p) in
          let replay_meta,ts,tr = extract_common p (List.length children) in
          let weights =
@@ -472,7 +470,6 @@ let () =
            if weights <> [] then weights else
              Utils.make_list (List.length children) 1
          in
-         let strict = Lang.to_bool (List.assoc "strict" p) in
          let children =
            if List.length weights <> List.length children then
              raise
@@ -486,4 +483,14 @@ let () =
                 (fun tr s -> { transition = tr ; source = s ; cur_meta = None })
                 tr children)
          in
-           ((new random ~replay_meta strict ts children):>source)) ;
+           new random ~replay_meta strict ts children)
+  in
+    add "random" false
+      "At the beginning of every track, select a random ready child."
+      "Weights of the children (padded with 1), defining for each child \
+       the probability that it is selected." ;
+    add "rotate" true
+      "Rotate between the sources."
+      "Weights of the children (padded with 1), defining for each child \
+       how many tracks are played from it per round, if that many are \
+       actually available."
