@@ -62,12 +62,14 @@ end
 (** Output in an MP3 file *)
 
 class to_file
+  ~infallible ~on_start ~on_stop
   ~append ~perm ~dir_perm
   ~reload_delay ~reload_predicate ~reload_on_metadata
   ~filename ~samplerate ~bitrate ~quality ~stereo ~autostart source =
 object (self)
   inherit
     Output.encoded ~name:filename ~kind:"output.file" ~autostart source
+                   ~infallible ~on_start ~on_stop
   inherit
      File_output.to_file ~reload_delay ~reload_predicate ~reload_on_metadata
                          ~append ~perm ~dir_perm filename
@@ -91,11 +93,7 @@ end
 
 let () =
   Lang.add_operator "output.file.mp3"
-    ([ "start",
-      Lang.bool_t, Some (Lang.bool true),
-      Some "Start output threads on operator initialization." ;
-
-      "samplerate",
+    (["samplerate",
       Lang.int_t,
       Some (Lang.int 44100),
       None ;
@@ -115,18 +113,28 @@ let () =
       Some (Lang.bool true),
       None]
 
-      @ File_output.proto @ ["", Lang.source_t, None, None ])
+      @ File_output.proto @ Output.proto @
+     ["", Lang.source_t, None, None ])
     ~category:Lang.Output
     ~descr:"Output the source's stream as an MP3 file."
-    (fun p ->
+    (fun p _ ->
        let e f v = f (List.assoc v p) in
        let quality = e Lang.to_int "quality" in
-       let autostart = e Lang.to_bool "start" in
        let stereo = e Lang.to_bool "stereo" in
        let samplerate = e Lang.to_int "samplerate" in
        let bitrate = e Lang.to_int "bitrate" in
        let filename = Lang.to_string (Lang.assoc "" 1 p) in
        let source = Lang.assoc "" 2 p in
+       let autostart = e Lang.to_bool "start" in
+       let infallible = not (Lang.to_bool (List.assoc "fallible" p)) in
+       let on_start =
+         let f = List.assoc "on_start" p in
+           fun () -> ignore (Lang.apply f [])
+       in
+       let on_stop =
+         let f = List.assoc "on_stop" p in
+           fun () -> ignore (Lang.apply f [])
+       in
        let append = Lang.to_bool (List.assoc "append" p) in
        let perm = Lang.to_int (List.assoc "perm" p) in
        let dir_perm = Lang.to_int (List.assoc "dir_perm" p) in
@@ -136,6 +144,7 @@ let () =
          Lang.to_bool (List.assoc "reopen_on_metadata" p)
        in
          ((new to_file ~filename
+             ~infallible ~on_start ~on_stop
              ~append ~perm ~dir_perm
              ~reload_delay ~reload_predicate ~reload_on_metadata
              ~quality ~bitrate ~samplerate ~stereo ~autostart source):>source))
