@@ -24,11 +24,43 @@
 
 open Source
 
+let proto = 
+ [
+  "fallible",
+  Lang.bool_t,
+  Some (Lang.bool false),
+  Some
+    "Allow the child source to fail, \
+     in which case the output will be (temporarily) stopped." ;
+
+  "on_start",
+  Lang.fun_t [] Lang.unit_t,
+  Some (Lang.val_cst_fun [] Lang.unit),
+  Some "Callback executed when outputting starts." ;
+
+  "on_stop",
+  Lang.fun_t [] Lang.unit_t,
+  Some (Lang.val_cst_fun [] Lang.unit),
+  Some "Callback executed when outputting stops."  ;
+
+  "start",
+  Lang.bool_t,
+  Some (Lang.bool true),
+  Some "Automatically start outputting whenever possible. \
+        If true, an infallible (normal) output will start outputting \
+        as soon as it is created, \
+        and a fallible output will (re)start as soon as its \
+        source becomes available for streaming."
+ ]
+
 (** Given abstract start stop and send methods, creates an output.
   * Takes care of pulling the data out of the source, type checkings,
   * maintains a queue of last ten metadata and setups standard Server commands,
   * including start/stop. *)
-class virtual output ~kind ?(name="") val_source autostart =
+class virtual output ~kind ?(name="")
+    ~infallible 
+    ~(on_start:unit->unit) ~(on_stop:unit->unit)
+    val_source autostart =
   let source = Lang.to_source val_source in
 object (self)
   inherit active_operator source as super
@@ -38,16 +70,25 @@ object (self)
   method virtual output_send : Frame.t -> unit
 
   initializer
-    (* We need the source to be infallible. *)
-    if source#stype <> Infallible then
+    if infallible && source#stype <> Infallible then
       raise (Lang.Invalid_value (val_source, "That source is fallible"))
 
+  method stype = source#stype
+
+  method is_ready =
+    if infallible then begin
+      assert (source#is_ready) ;
+      true
+    end else
+      source#is_ready
+
   method remaining = source#remaining
   method abort_track = source#abort_track
 
   val mutable does_output = false       (* Currently outputting *)
   val mutable start_output = autostart  (* Ask for startup *)
   val mutable stop_output = false       (* Ask for termination *)
+  val mutable autostart = autostart     (* Start as soon as possible *)
 
   method is_active = does_output
 
@@ -83,29 +124,59 @@ object (self)
            if r < 0 then "(undef)" else
              let t = Fmt.seconds_of_ticks r in
                Printf.sprintf "%.2f" t) ;
-    Server.add ~ns "start" (fun _ -> start_output <- true ; "") 
-               ~descr:"Start output." ;
-    Server.add ~ns "stop" (fun _ -> stop_output <- true ; "") 
-               ~descr:"Stop output."  ;
-    Server.add ~ns "status" (fun _ -> if does_output then "on" else "off") 
-               ~descr:"Get status.";
+    Server.add ~ns "autostart" ~descr:"Enable/disable autostart."
+      (fun s ->
+         if s <> "" then begin
+           let update = s = "on" || s = "yes" || s = "y" in
+             (* Update start_output when:
+              *  - autostart becomes true (we now wait to start asap)
+              *  - autostart becomes false too (stop ongoing waiting)
+              * But not when it is unchanged. For example, this prevents
+              * cancelling a manually-ordered start. *)
+             if update <> autostart then begin
+               start_output <- update ;
+               autostart <- update
+             end
+         end ;
+         if autostart then "on" else "off") ;
+    Server.add ~ns "start" ~descr:"Start output."
+      (fun _ -> start_output <- true ; "OK") ;
+    Server.add ~ns "stop" ~descr:"Stop output. Disable autostart."
+      (fun _ ->
+         if autostart then begin
+           autostart <- false ;
+           start_output <- false
+         end ;
+         stop_output <- true ;
+         "OK") ;
+    Server.add ~ns "status" ~descr:"Get status."
+      (fun _ -> if does_output then "on" else "off") ;
 
-    (* Get our source ready *)
+    (* Get our source ready.
+     * This can take a while (preparing playlists, etc). *)
     source#get_ready ((self:>operator)::activation) ;
-    while not source#is_ready do
-      self#log#f 3 "Waiting for %S to be ready..." source#id ;
-      Thread.delay 1. ;
-    done
+    if infallible then
+      while not source#is_ready do
+        self#log#f 3 "Waiting for %S to be ready..." source#id ;
+        Thread.delay 1. ;
+      done
 
   method output_get_ready =
-    if start_output then begin
-      start_output <- false ;
-      if not does_output then self#output_start ;
-      does_output <- true
+    if start_output && self#is_ready then begin
+      start_output <- autostart ;
+      if not does_output then begin
+        self#output_start ;
+        on_start () ;
+        does_output <- true
+      end
     end
 
   method private sleep =
-    if does_output then self#output_stop ;
+    if does_output then begin
+      self#output_stop ;
+      on_stop () ;
+      does_output <- false
+    end ;
     source#leave (self:>operator)
 
   (* Metadata stuff: keep track of what was streamed. *)
@@ -126,15 +197,18 @@ object (self)
   method private get_frame buf = source#get buf
 
   method private output =
-    if start_output then begin
-      start_output <- false ;
-      if not does_output then self#output_start ;
-      does_output <- true
+    if start_output && self#is_ready then begin
+      start_output <- autostart ;
+      if not does_output then begin
+        self#output_start ;
+        on_start () ;
+        does_output <- true
+      end
     end ;
 
     if does_output then begin
       (* Complete filling of the frame *)
-      while Frame.is_partial memo do
+      while Frame.is_partial memo && self#is_ready do
         source#get memo
       done ;
       List.iter
@@ -144,10 +218,15 @@ object (self)
       self#output_send memo
     end ;
 
-    if stop_output then begin
+    if stop_output || Frame.is_partial memo then begin
       stop_output <- false ;
-      if does_output then self#output_stop ;
-      does_output <- false
+      if does_output then begin
+        if Frame.is_partial memo then
+          self#log#f 3 "Source failed (no more tracks), output stops." ;
+        self#output_stop ;
+        on_stop () ;
+        does_output <- false
+      end
     end
 
   method after_output =
@@ -162,25 +241,46 @@ object (self)
 
 end
 
-class dummy source = object
-  inherit output ~kind:"output.dummy" source true
+class dummy ~infallible ~on_start ~on_stop ~autostart source = object
+  inherit output ~kind:"output.dummy" source autostart
+                 ~infallible ~on_start ~on_stop
   method output_reset  = ()
   method output_start  = ()
   method output_stop   = ()
   method output_send _ = ()
 end
+
 let () =
   Lang.add_operator "output.dummy"
-    ["",Lang.source_t,None,None]
+    ["fallible", Lang.bool_t, Some (Lang.bool false), None;
+     "autostart", Lang.bool_t, Some (Lang.bool true), None;
+     "on_start", Lang.fun_t [] Lang.unit_t,
+     Some (Lang.val_cst_fun [] Lang.unit), None ;
+     "on_stop", Lang.fun_t [] Lang.unit_t,
+     Some (Lang.val_cst_fun [] Lang.unit), None ;
+     "", Lang.source_t, None, None]
     ~category:Lang.Output
     ~descr:"Dummy output for debugging purposes."
-    (fun p -> ((new dummy (List.assoc "" p)):>Source.source))
+    (fun p _ ->
+       let infallible = not (Lang.to_bool (List.assoc "fallible" p)) in
+       let autostart = Lang.to_bool (List.assoc "autostart" p) in
+       let on_start = List.assoc "on_start" p in
+       let on_stop = List.assoc "on_stop" p in
+       let on_start () = ignore (Lang.apply on_start []) in
+       let on_stop () = ignore (Lang.apply on_stop []) in
+         ((new dummy ~on_start ~on_stop ~infallible ~autostart
+             (List.assoc "" p)):>Source.source))
 
 (** More concrete abstract-class, which takes care of the #output_send
   * method for outputs based on encoders. *)
-class virtual encoded ~kind ~name ~autostart source =
+class virtual encoded
+  ~kind ~name
+  ~infallible ~on_start ~on_stop
+  ~autostart source =
 object (self)
-  inherit output ~kind ~name source autostart
+  inherit output
+            ~infallible ~on_start ~on_stop
+            ~kind ~name source autostart
 
   method virtual reset_encoder : (string,string) Hashtbl.t -> string
   method virtual encode : Frame.t -> int -> int -> string
@@ -203,7 +303,7 @@ object (self)
       in
         function
           | [] -> assert false
-          | [i] -> assert (i=AFrame.size frame)
+          | [i] -> assert (i=AFrame.size frame || not infallible)
           | start::stop::l ->
               if start < stop then f start stop else assert (start=stop) ;
               output_chunks frame (stop::l)
