@@ -95,11 +95,8 @@ let create ~quality ~mode ~bitrate samplerate stereo =
 
 let () =
   Lang.add_operator "output.file.vorbis.abr" (* Average BitRate *)
-    (vorbis_proto @ File_output.proto @ [
-      "start",
-      Lang.bool_t, Some (Lang.bool true),
-      Some "Start output on operator initialization." ;
-
+    (vorbis_proto @ File_output.proto @ Output.proto @
+     [
       "bitrate",
       Lang.int_t,
       Some (Lang.int 128),
@@ -119,9 +116,8 @@ let () =
     ~category:Lang.Output
     ~descr:("Output the source stream as an Ogg Vorbis file "
             ^ "in Average BitRate mode.")
-    (fun p ->
+    (fun p _ ->
        let e f v = f (List.assoc v p) in
-       let autostart = e Lang.to_bool "start" in
        let stereo = e Lang.to_bool "stereo" in
        let skeleton = e Lang.to_bool "skeleton" in
        let bitrate = (e Lang.to_int "bitrate") * 1000 in
@@ -137,13 +133,23 @@ let () =
        let reload_on_metadata =
          Lang.to_bool (List.assoc "reopen_on_metadata" p)
        in
+       let autostart = e Lang.to_bool "start" in
+       let infallible = not (Lang.to_bool (List.assoc "fallible" p)) in
+       let on_start =
+         let f = List.assoc "on_start" p in
+           fun () -> ignore (Lang.apply f [])
+       in
+       let on_stop =
+         let f = List.assoc "on_stop" p in
+           fun () -> ignore (Lang.apply f [])
+       in
        let source = Lang.assoc "" 2 p in
        let streams = 
          ["vorbis",create ~quality:0. ~mode:ABR 
                           ~bitrate:(bitrate, min_bitrate, max_bitrate) 
                           freq stereo]
        in
-         ((new Ogg_output.to_file
+         ((new Ogg_output.to_file ~infallible ~on_stop ~on_start
              name ~append ~perm ~dir_perm ~streams ~skeleton
              ~reload_delay ~reload_predicate ~reload_on_metadata
              ~autostart source):>source))
@@ -158,12 +164,12 @@ let () =
       "bitrate",
       Lang.int_t,
       Some (Lang.int 128),
-      Some "Bitrate (in kbps)."] @ File_output.proto @
+      Some "Bitrate (in kbps)."] @ File_output.proto @ Output.proto @
       ["", Lang.source_t, None, None ])
     ~category:Lang.Output
     ~descr:("Output the source stream as an Ogg Vorbis file "
             ^ "in Constant BitRate mode.")
-    (fun p ->
+    (fun p _ ->
        let e f v = f (List.assoc v p) in
        let autostart = e Lang.to_bool "start" in
        let stereo = e Lang.to_bool "stereo" in
@@ -184,8 +190,17 @@ let () =
                           ~bitrate:(bitrate, bitrate, bitrate)
                           freq stereo]
        in
+       let infallible = not (Lang.to_bool (List.assoc "fallible" p)) in
+       let on_start =
+         let f = List.assoc "on_start" p in
+           fun () -> ignore (Lang.apply f [])
+       in
+       let on_stop =
+         let f = List.assoc "on_stop" p in
+           fun () -> ignore (Lang.apply f [])
+       in
        let source = Lang.assoc "" 2 p in
-         ((new Ogg_output.to_file
+         ((new Ogg_output.to_file ~infallible ~on_stop ~on_start
              name ~append ~perm ~dir_perm ~streams ~skeleton
              ~reload_delay ~reload_predicate ~reload_on_metadata
              ~autostart source):>source))
@@ -202,11 +217,12 @@ let () =
       Some (Lang.float 2.),
       Some ("Desired quality level, currently from -1. to 10. (low to high).")] 
 
-      @ File_output.proto @ ["", Lang.source_t, None, None ])
+      @ File_output.proto @ Output.proto @
+     ["", Lang.source_t, None, None ])
     ~category:Lang.Output
     ~descr:("Output the source stream as an Ogg Vorbis file "
             ^ "in Variable BitRate mode.")
-    (fun p ->
+    (fun p _ ->
        let e f v = f (List.assoc v p) in
        let autostart = e Lang.to_bool "start" in
        let stereo = e Lang.to_bool "stereo" in
@@ -214,6 +230,15 @@ let () =
        let quality = (e Lang.to_float "quality") *. 0.1 in
        let freq = e Lang.to_int "samplerate" in
        let name = Lang.to_string (Lang.assoc "" 1 p) in
+       let infallible = not (Lang.to_bool (List.assoc "fallible" p)) in
+       let on_start =
+         let f = List.assoc "on_start" p in
+           fun () -> ignore (Lang.apply f [])
+       in
+       let on_stop =
+         let f = List.assoc "on_stop" p in
+           fun () -> ignore (Lang.apply f [])
+       in
        let append = Lang.to_bool (List.assoc "append" p) in
        let perm = Lang.to_int (List.assoc "perm" p) in
        let dir_perm = Lang.to_int (List.assoc "dir_perm" p) in
@@ -230,6 +255,6 @@ let () =
        in
          ((new Ogg_output.to_file
              name ~append ~perm ~dir_perm ~streams ~skeleton 
+             ~infallible ~on_start ~on_stop
              ~reload_delay ~reload_predicate ~reload_on_metadata
              ~autostart source):>source))
-
