@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2008 Savonet team
+  Copyright 2003-2009 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -30,8 +30,8 @@ val running : unit -> bool
 (** Set [shutdown] to false to stop the loop *)
 val shutdown : bool ref
 
-(** Forces the shutdown of the scheduler, in case the root's thread died. *)
-val force_sleep : unit -> unit
+(** Puts the root sources to sleep, to do only if the scheduler crashed. *)
+val sleep : unit -> unit
 
 val uptime : unit -> float
 val delay : unit -> float
