@@ -290,6 +290,7 @@ object (self)
               | Some f -> close_out f ; logf <- None
               | None -> ()
             end ;
+            Http.disconnect socket ;
             if debug then raise e
 
   method connect = self#private_connect ~encode:true
@@ -366,6 +367,7 @@ object (self)
                 raise Internal
               end ;
               let play_track (m,uri) =
+                if not poll_should_stop then
                 let metas = Hashtbl.create 2 in
                   List.iter (fun (a,b) -> Hashtbl.add metas a b) m;
                   self#insert_metadata metas;
@@ -391,8 +393,7 @@ object (self)
                 let playlist = parser content in
                   match playlist with
                     | [] -> raise Not_found
-                    | _ -> () ;
-                           playlist_process playlist
+                    | _ -> playlist_process playlist
               in
                 try
                   self#log#f 4
@@ -405,37 +406,38 @@ object (self)
                   | Not_found ->
                       (* Trying playlist auto parsing in case
                        * of content type text/plain *)
-                      if content_type = "text/plain" then
-                        begin
-                          try
-                            test_playlist
-                              (fun x -> snd (Playlist_parser.search_valid x))
-                          with
-                            | Not_found -> ()
-                        end;
-                      self#log#f 4 "Content-type \"%s\"." content_type ;
-                      if chunked then self#log#f 4 "Chunked HTTP/1.1 transfer" ;
-                      let dec =
-                        match
-                          stream_decoders#get content_type
+                      if content_type = "text/plain" then begin
+                        try
+                          test_playlist
+                            (fun x -> snd (Playlist_parser.search_valid x))
                         with
-                          | Some d -> d
-                          | None -> failwith "Unknown format!"
-                      in
-                        begin match logfile with
-                          | Some f ->
-                              begin try
-                                logf <-
-                                Some (open_out_bin (Utils.home_unrelate f))
-                              with e ->
-                                self#log#f 2
-                                  "Could not open log file: %s"
-                                  (Printexc.to_string e)
-                              end
-                          | None -> ()
-                        end ;
-                        self#log#f 3 "Decoding..." ;
-                        self#feeding dec socket chunked metaint
+                          | Not_found -> ()
+                      end else begin
+                        self#log#f 4 "Content-type \"%s\"." content_type ;
+                        if chunked then
+                          self#log#f 4 "Chunked HTTP/1.1 transfer" ;
+                        let dec =
+                          match
+                            stream_decoders#get content_type
+                          with
+                            | Some d -> d
+                            | None -> failwith "Unknown format!"
+                        in
+                          begin match logfile with
+                            | Some f ->
+                                begin try
+                                  logf <-
+                                  Some (open_out_bin (Utils.home_unrelate f))
+                                with e ->
+                                  self#log#f 2
+                                    "Could not open log file: %s"
+                                    (Printexc.to_string e)
+                                end
+                            | None -> ()
+                          end ;
+                          self#log#f 3 "Decoding..." ;
+                          self#feeding dec socket chunked metaint
+                      end
           with
             | e ->
                 Http.disconnect socket;
@@ -456,7 +458,6 @@ object (self)
       poll_should_stop <- false ;
       Condition.signal polling_cond
     end else begin
-      (* TODO Use Duppy instead of a separate thread ? *)
       Thread.delay poll_delay ;
       self#poll
     end
@@ -473,7 +474,12 @@ object (self)
     Server.add ~ns "start" ~usage:"start" ~descr:"Start the source, if needed."
        (fun _ -> relaying <- true ; "Done") ;
     Server.add ~ns "stop" ~usage:"stop" ~descr:"Stop the source if streaming."
-       (fun _ -> relaying <- false ; "Done")
+       (fun _ -> relaying <- false ; "Done") ;
+    Server.add ~ns "buffer_length" ~usage:"buffer_length" 
+               ~descr:"Get the buffer's length, in seconds."
+       (fun _ -> Printf.sprintf "%.2f" 
+             (Fmt.seconds_of_samples self#length))
+
 
   method sleep = poll_should_stop <- true
 
@@ -532,7 +538,7 @@ let () =
 
         "", Lang.string_t, None,
         Some "URL of an http stream (default port is 80)." ]
-      (fun p ->
+      (fun p _ ->
          let playlist_mode =
            let s = List.assoc "playlist_mode" p in
              match Lang.to_string s with
@@ -573,6 +579,10 @@ let () =
          let bufferize = Lang.to_float (List.assoc "buffer" p) in
          let timeout = Lang.to_float (List.assoc "timeout" p) in
          let max = Lang.to_float (List.assoc "max" p) in
+         if bufferize > max then
+           raise (Lang.Invalid_value
+                    (List.assoc "max" p,
+                     "Maximun buffering inferior to pre-buffered data"));
          let poll_delay = Lang.to_float (List.assoc "poll_delay" p) in
            ((new http ~playlist_mode ~timeout ~autostart ~track_on_meta
                       ~force_mime ~bind_address ~poll_delay
