@@ -43,16 +43,6 @@ let to_s16le b =
     assert (Float_pcm.to_s16le fpcm 0 (Array.length fpcm.(0)) s 0 = slen);
     s
 
-let fill_frame gen frame =
-  add_break frame
-    (Float_pcm.Generator.fill
-       gen (get_float_pcm frame) (position frame))
-
-let fill_frame_from_raw gen frame =
-  add_break frame
-    (Float_pcm.Generator_from_raw.fill
-       gen (get_float_pcm frame) (position frame))
-
 let blankify b off len =
   Float_pcm.blankify (get_float_pcm b) off len
 
