@@ -56,11 +56,7 @@ val get_s16le_length : t -> int
 
 val to_s16le : t -> string
 
-(** Fill a frame from a (float) generator. *)
-val fill_frame : Float_pcm.Generator.t -> Frame.t -> unit
-
-(** Fill a frame from a (raw) generator. *)
-val fill_frame_from_raw : Float_pcm.Generator_from_raw.t -> Frame.t -> unit
+(** {2 Sound processing} *)
 
 val blankify : t -> int -> int -> unit
 
