@@ -1,7 +1,7 @@
 /*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2008 Savonet team
+  Copyright 2003-2009 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -36,8 +36,9 @@
 #include <caml/mlvalues.h>
 
 #include <limits.h>
+#include <stdint.h>
 #include "config.h"
-static inline unsigned short bswap_16(unsigned short x) { return (x>>8) | (x<<8); }
+static inline int16_t bswap_16 (int16_t x) { return ((((x) >> 8) & 0xff) | (((x) & 0xff) << 8)); }
 
 #include <assert.h>
 #include <stdio.h>
@@ -107,7 +108,9 @@ CAMLprim value caml_float_pcm_to_s16le(value a, value _offs, value _len, value _
   CAMLreturn(Val_int(dst_len));
 }
 
-CAMLprim value caml_float_pcm_convert_native(
+#define s2f(x) (((double)x)/32768)
+
+CAMLprim value caml_float_pcm_convert_le_native(
     value _src, value _offset, value _length,
     value _signed, value _bits_per_sample, value _bigendian,
     value _ratio,
@@ -133,25 +136,24 @@ CAMLprim value caml_float_pcm_convert_native(
   // TODO get_s16le get_s16be get_s32le etc.
   //      correspondingly there should be convert_s16le, and others, expanded
   //      from macros.
-#define s2f(x) (((double)x)/32768)
 #ifdef LIQ_BIG_ENDIAN
-#define get_sample(c,i) s2f(bswap_16(((short*)src)[offset/2+i*nc+c]))
+#define get_sample_le(c,i) s2f(bswap_16(((int16_t*)src)[offset/2+i*nc+c]))
 #else
-#define get_sample(c,i) s2f(((short*)src)[offset/2+i*nc+c])
+#define get_sample_le(c,i) s2f(((int16_t*)src)[offset/2+i*nc+c])
 #endif
 
   if (ratio==1) {
     for (c=0 ; c<nc ; c++) {
       dstc = Field(_dst,c) ;
       for (i=0 ; i<newlen; i++) {
-        Store_double_field(dstc, dst_off+i, get_sample(c,i)) ;
+        Store_double_field(dstc, dst_off+i, get_sample_le(c,i)) ;
       }
     }
   }else{
     for (c=0 ; c<nc ; c++) {
       dstc = Field(_dst,c) ;
       for (i=0 ; i<newlen; i++) {
-        Store_double_field(dstc, dst_off+i, get_sample(c,((int)(i/ratio)))) ;
+        Store_double_field(dstc, dst_off+i, get_sample_le(c,((int)(i/ratio)))) ;
       }
     }
   }
@@ -159,8 +161,65 @@ CAMLprim value caml_float_pcm_convert_native(
   CAMLreturn(Val_int(dst_off+newlen)) ;
 }
 
-CAMLprim value caml_float_pcm_convert_byte(value* argv, int argn) {
-  return caml_float_pcm_convert_native(argv[0],argv[1],argv[2],
+CAMLprim value caml_float_pcm_convert_le_byte(value* argv, int argn) {
+  return caml_float_pcm_convert_le_native(argv[0],argv[1],argv[2],
+                                       argv[3],argv[4],argv[5],
+                                       argv[6],argv[7],argv[8]) ;
+}
+
+CAMLprim value caml_float_pcm_convert_be_native(
+    value _src, value _offset, value _length,
+    value _signed, value _bits_per_sample, value _bigendian,
+    value _ratio,
+    value _dst, value _dst_off)
+{
+  CAMLparam2(_src, _dst) ;
+  CAMLlocal1(dstc) ;
+  char* src = String_val(_src) ;
+  int offset = Int_val(_offset) ;
+  int len = Int_val(_length) ;
+  double ratio = Double_val(_ratio) ;
+  int dst_off = Int_val(_dst_off) ;
+  int dst_len = Wosize_val(Field(_dst, 0)) / Double_wosize ;
+  int newlen = (int)(ratio*len) ;
+  int i,c ;
+  int nc = Wosize_val(_dst) ;
+
+  if (dst_off + newlen > dst_len)
+    caml_invalid_argument("convert_native: output buffer too small");
+  if (Int_val(_bits_per_sample) != 16 || !Bool_val(_signed) || !Bool_val(_bigendian))
+    caml_invalid_argument("convert_native: bad input format");
+
+  // TODO get_s16le get_s16be get_s32le etc.
+  //      correspondingly there should be convert_s16le, and others, expanded
+  //      from macros.
+#ifdef LIQ_BIG_ENDIAN
+#define get_sample_be(c,i) s2f(((int16_t*)src)[offset/2+i*nc+c])
+#else
+#define get_sample_be(c,i) s2f(bswap_16(((int16_t*)src)[offset/2+i*nc+c]))
+#endif
+
+  if (ratio==1) {
+    for (c=0 ; c<nc ; c++) {
+      dstc = Field(_dst,c) ;
+      for (i=0 ; i<newlen; i++) {
+        Store_double_field(dstc, dst_off+i, get_sample_be(c,i)) ;
+      }
+    }
+  }else{
+    for (c=0 ; c<nc ; c++) {
+      dstc = Field(_dst,c) ;
+      for (i=0 ; i<newlen; i++) {
+        Store_double_field(dstc, dst_off+i, get_sample_be(c,((int)(i/ratio)))) ;
+      }
+    }
+  }
+
+  CAMLreturn(Val_int(dst_off+newlen)) ;
+}
+
+CAMLprim value caml_float_pcm_convert_be_byte(value* argv, int argn) {
+  return caml_float_pcm_convert_be_native(argv[0],argv[1],argv[2],
                                        argv[3],argv[4],argv[5],
                                        argv[6],argv[7],argv[8]) ;
 }
