@@ -940,7 +940,7 @@ CAMLprim value caml_rgb_disk_opacity(value _rgb, value _x, value _y, value _r)
   for (j = 0; j < rgb.height; j++)
     for (i = 0; i < rgb.width; i++)
     {
-      r = sqrt((i - x) * (i - x) + (j - y) * (j - y));
+      r = sqrt((double)(i - x) * (i - x) + (j - y) * (j - y));
       if (r > radius)
         Alpha(&rgb, i, j) = 0;
     }
