@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2008 Savonet team
+  Copyright 2003-2009 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -31,8 +31,11 @@ type t = {
  * wpos stores the current write position *)
 
 let create chans size =
-  { size = size ;
-    buffer = Array.init chans (fun _ -> Array.create size 0.) ;
+  {
+    (* size + 1 so we can store full buffers, while keeping
+       rpos and wpos different for implementation matters *)
+    size = size + 1 ;
+    buffer = Array.init chans (fun _ -> Array.create (size + 1) 0.) ;
     rpos = 0 ;
     wpos = 0
   }
@@ -59,35 +62,31 @@ let read t buff off len =
   assert (len <= read_space t) ;
   let pre = t.size - t.rpos in
   let extra = len - pre in
-  begin match extra > 0 with
-  | false ->
-      for c = 0 to Array.length t.buffer - 1 do
-        Array.blit t.buffer.(c) t.rpos buff.(c) off len
-      done;
-  | true ->
+    if extra > 0 then
       for c = 0 to Array.length t.buffer - 1 do
         Array.blit t.buffer.(c) t.rpos buff.(c) off pre;
         Array.blit t.buffer.(c) 0 buff.(c) (off + pre) extra
+      done
+    else
+      for c = 0 to Array.length t.buffer - 1 do
+        Array.blit t.buffer.(c) t.rpos buff.(c) off len
       done;
-  end;
-  read_advance t len
+    read_advance t len
 
 let write t buff off len =
   assert (len <= write_space t) ;
   let pre = t.size - t.wpos in
   let extra = len - pre in
-  begin match extra > 0 with
-  | false ->
-      for c = 0 to Array.length t.buffer - 1 do
-        Array.blit buff.(c) off t.buffer.(c) t.wpos len
-      done;
-  | true ->
+    if extra > 0 then
       for c = 0 to Array.length t.buffer - 1 do
         Array.blit buff.(c) off t.buffer.(c) t.wpos pre;
         Array.blit buff.(c) (off + pre) t.buffer.(c) 0 extra
+      done
+    else
+      for c = 0 to Array.length t.buffer - 1 do
+        Array.blit buff.(c) off t.buffer.(c) t.wpos len
       done;
-  end;
-  write_advance t len
+    write_advance t len
 
 let transmit t f =
   if t.wpos = t.rpos then 0 else
@@ -99,3 +98,45 @@ let transmit t f =
   assert (len <= len0) ;
   read_advance t len ;
   len
+
+let ts m f =
+  try
+    Mutex.lock m;
+    let ans = f () in
+      Mutex.unlock m;
+      ans
+  with
+    | e ->
+        Mutex.unlock m;
+        raise e
+
+module TS =
+struct
+  type ts_t = t * Mutex.t
+
+  type t = ts_t
+
+  let create chans size =
+    create chans size, Mutex.create ()
+
+  let read_space (t,m) =
+    ts m (fun () -> read_space t)
+
+  let write_space (t,m) =
+    ts m (fun () -> write_space t)
+
+  let read_advance (t,m) n =
+    ts m (fun () -> read_advance t n)
+
+  let write_advance (t,m) n =
+    ts m (fun () -> write_advance t n)
+
+  let read (t,m) buf off len =
+    ts m (fun () -> read t buf off len)
+
+  let write (t,m) buf off len =
+    ts m (fun () -> write t buf off len)
+
+  let transmit (t,m) f =
+    ts m (fun () -> transmit t f)
+end
