@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2008 Savonet team
+  Copyright 2003-2009 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -33,10 +33,10 @@ type t
   * per channel. *)
 val create : int -> int -> t
 
-(** Get the length of available data. *)
+(** Get the length of readable space. *)
 val read_space : t -> int
 
-(** Get the length of available read space. *)
+(** Get the length of writable space. *)
 val write_space : t -> int
 
 (** Advance the read pointer. *)
@@ -62,3 +62,25 @@ val write : t -> float array array -> int -> int -> unit
   * ringbuffer [rb] through [f] and returns the length of the chunk actually
   * read by [f]. *)
 val transmit : t -> (float array array -> int -> int -> int) -> int
+
+(** Thread-safe interface. *)
+module TS :
+sig
+  type t
+
+  val create : int -> int -> t
+
+  val read_space : t -> int
+
+  val write_space : t -> int
+
+  val read_advance : t -> int -> unit
+
+  val write_advance : t -> int -> unit
+
+  val read : t -> float array array -> int -> int -> unit
+
+  val write : t -> float array array -> int -> int -> unit
+
+  val transmit : t -> (float array array -> int -> int -> int) -> int
+end
